@@ -223,7 +223,7 @@ def install(args, packages, suffix="native", build=True):
     for command in commands:
         if args.offline:
             command = ["--no-network"] + command
-        pmb.chroot.root(args, ["apk", "--no-progress"] + command, suffix)
+        pmb.chroot.root(args, ["apk", "--no-progress"] + command, suffix=suffix, disable_timeout=True)
 
 
 def upgrade(args, suffix="native"):
