@@ -14,7 +14,7 @@ from pmb.config.merge_with_args import merge_with_args
 #
 # Exported variables (internal configuration)
 #
-version = "1.18.1"
+version = "1.21.0"
 pmb_src = os.path.normpath(os.path.realpath(__file__) + "/../../..")
 apk_keys_path = pmb_src + "/pmb/data/keys"
 
@@ -24,12 +24,15 @@ apk_keys_path = pmb_src + "/pmb/data/keys"
 apk_tools_static_min_version = "2.10.5-r0"
 
 # postmarketOS aports compatibility (checked against "version" in pmaports.cfg)
-pmaports_min_version = "6"
+pmaports_min_version = "7"
 
 # Version of the work folder (as asked during 'pmbootstrap init'). Increase
 # this number, whenever migration is required and provide the migration code,
 # see migrate_work_folder()).
-work_version = 4
+work_version = 5
+
+# Minimum required version of postmarketos-ondev (pmbootstrap install --ondev)
+ondev_min_version = "0.2.0"
 
 # Programs that pmbootstrap expects to be available from the host system. Keep
 # in sync with README.md, and try to keep the list as small as possible. The
@@ -37,20 +40,33 @@ work_version = 4
 required_programs = ["git", "openssl", "ps"]
 
 # Keys saved in the config file (mostly what we ask in 'pmbootstrap init')
-config_keys = ["ccache_size", "device", "extra_packages", "hostname", "jobs",
-               "kernel", "keymap", "nonfree_firmware", "nonfree_userland",
-               "ssh_keys", "timezone", "ui", "user", "work", "aports"]
+config_keys = ["aports",
+               "ccache_size",
+               "device",
+               "extra_packages",
+               "hostname",
+               "is_default_channel",
+               "jobs",
+               "kernel",
+               "keymap",
+               "nonfree_firmware",
+               "nonfree_userland",
+               "ssh_keys",
+               "timezone",
+               "ui",
+               "ui_extras",
+               "user",
+               "work",
+               "boot_size"]
 
 # Config file/commandline default values
 # $WORK gets replaced with the actual value for args.work (which may be
 # overridden on the commandline)
 defaults = {
-    "alpine_version": "edge",  # alternatively: latest-stable
     "aports": "$WORK/cache_git/pmaports",
     "ccache_size": "5G",
-    # aes-xts-plain64 would be better, but this is not supported on LineageOS
-    # kernel configs
-    "cipher": "aes-cbc-plain64",
+    "is_default_channel": True,
+    "cipher": "aes-xts-plain64",
     "config": os.path.expanduser("~") + "/.config/pmbootstrap.cfg",
     "device": "qemu-amd64",
     "extra_packages": "none",
@@ -65,15 +81,17 @@ defaults = {
     "keymap": "",
     "log": "$WORK/log.txt",
     "mirror_alpine": "http://dl-cdn.alpinelinux.org/alpine/",
-    "mirrors_postmarketos": ["http://postmarketos1.brixit.nl/postmarketos/master"],
+    "mirrors_postmarketos": ["http://postmarketos1.brixit.nl/postmarketos/"],
     "nonfree_firmware": True,
     "nonfree_userland": False,
     "port_distccd": "33632",
     "ssh_keys": False,
     "timezone": "GMT",
     "ui": "weston",
+    "ui_extras": False,
     "user": "user",
     "work": os.path.expanduser("~") + "/.local/var/pmbootstrap",
+    "boot_size": "128",
 }
 
 #
@@ -103,6 +121,7 @@ chroot_host_path = os.environ["PATH"] + ":/usr/sbin/"
 # Folders, that get mounted inside the chroot
 # $WORK gets replaced with args.work
 # $ARCH gets replaced with the chroot architecture (eg. x86_64, armhf)
+# $CHANNEL gets replaced with the release channel (e.g. edge, stable)
 chroot_mount_bind = {
     "/proc": "/proc",
     "$WORK/cache_apk_$ARCH": "/var/cache/apk",
@@ -112,7 +131,7 @@ chroot_mount_bind = {
     "$WORK/cache_rust": "/mnt/pmbootstrap-rust",
     "$WORK/config_abuild": "/mnt/pmbootstrap-abuild-config",
     "$WORK/config_apk_keys": "/etc/apk/keys",
-    "$WORK/packages": "/mnt/pmbootstrap-packages",
+    "$WORK/packages/$CHANNEL": "/mnt/pmbootstrap-packages",
 }
 
 # Building chroots (all chroots, except for the rootfs_ chroot) get symlinks in
@@ -163,10 +182,6 @@ build_device_architectures = ["armhf", "armv7", "aarch64", "x86_64", "x86"]
 # for the first time
 build_packages = ["abuild", "build-base", "ccache", "git"]
 
-# fnmatch for supported pkgnames, that can be directly compiled inside
-# the native chroot and a cross-compiler, without using distcc
-build_cross_native = ["linux-*", "arch-bin-masquerade"]
-
 # Necessary kernel config options
 necessary_kconfig_options = {
     ">=0.0.0": {  # all versions
@@ -174,14 +189,15 @@ necessary_kconfig_options = {
             "ANDROID_PARANOID_NETWORK": False,
             "BLK_DEV_INITRD": True,
             "CGROUPS": True,
+            "CRYPTO_XTS": True,
             "DEVTMPFS": True,
             "DM_CRYPT": True,
             "EXT4_FS": True,
             "KINETO_GAN": False,
             "PFT": False,
             "SYSVIPC": True,
-            "VT": True,
             "USE_VFB": False,
+            "VT": True,
         }
     },
     ">=4.0.0": {
@@ -196,6 +212,32 @@ necessary_kconfig_options = {
     }
 }
 
+# Necessary anbox kernel config options
+necessary_kconfig_options_anbox = {
+    ">=0.0.0": {  # all versions
+        "all": {  # all arches
+            "SQUASHFS": True,
+            "SQUASHFS_XZ": True,
+            "SQUASHFS_XATTR": True,
+            "TMPFS_XATTR": True,
+            "ASHMEM": True,
+            "ANDROID_BINDER_IPC": True,
+            "ANDROID_BINDERFS": False,
+            "ANDROID_BINDER_DEVICES": ["binder", "hwbinder"],
+            "NETFILTER_XTABLES": True,
+            "NETFILTER_XT_MATCH_COMMENT": True,
+            "IP_NF_MANGLE": True,
+            "FUSE_FS": True,
+            "BLK_DEV_LOOP": True,
+            "TUN": True,
+            "VETH": True,
+            "VLAN_8021Q": True,  # prerequisite for bridge
+            "BRIDGE": True,
+            "BRIDGE_VLAN_FILTERING": True,
+        }
+    },
+}
+
 #
 # PARSE
 #
@@ -244,6 +286,15 @@ apkbuild_attributes = {
     "source": {"array": True},
 }
 
+# Reference: https://postmarketos.org/apkbuild-options
+apkbuild_custom_valid_options = [
+    "!pmb:crossdirect",
+    "!pmb:kconfig-check",
+    "pmb:kconfigcheck-anbox",
+    "pmb:cross-native",
+    "pmb:strict",
+]
+
 # Variables from deviceinfo. Reference: <https://postmarketos.org/deviceinfo>
 deviceinfo_attributes = [
     # general
@@ -257,6 +308,7 @@ deviceinfo_attributes = [
     "arch",
 
     # device
+    "chassis",
     "keyboard",
     "external_storage",
     "screen_width",
@@ -272,6 +324,7 @@ deviceinfo_attributes = [
     "flash_heimdall_partition_kernel",
     "flash_heimdall_partition_initfs",
     "flash_heimdall_partition_system",
+    "flash_heimdall_partition_vbmeta",
     "flash_fastboot_partition_kernel",
     "flash_fastboot_partition_system",
     "flash_fastboot_partition_vbmeta",
@@ -301,6 +354,20 @@ deviceinfo_attributes = [
     "keymaps",
 ]
 
+# Valid types for the 'chassis' atribute in deviceinfo
+# See https://www.freedesktop.org/software/systemd/man/machine-info.html
+deviceinfo_chassis_types = [
+    "desktop",
+    "laptop",
+    "convertible",
+    "server",
+    "tablet",
+    "handset",
+    "watch",
+    "embedded",
+    "vm"
+]
+
 #
 # INITFS
 #
@@ -409,7 +476,7 @@ flashers = {
     # Some Samsung devices need a 'boot.img' file, just like the one generated
     # fastboot compatible devices. Example: s7562, n7100
     "heimdall-bootimg": {
-        "depends": ["heimdall"],
+        "depends": ["heimdall", "avbtool"],
         "actions": {
             "list_devices": [["heimdall", "detect"]],
             "flash_rootfs": [
@@ -418,6 +485,12 @@ flashers = {
             "flash_kernel": [
                 ["heimdall_wait_for_device.sh"],
                 ["heimdall", "flash", "--$PARTITION_KERNEL", "$BOOT/boot.img-$FLAVOR"]],
+            "flash_vbmeta": [
+                ["avbtool", "make_vbmeta_image", "--flags", "2",
+                    "--padding_size", "$FLASH_PAGESIZE",
+                    "--output", "/vbmeta.img"],
+                ["heimdall", "flash", "--$PARTITION_VBMETA", "/vbmeta.img"],
+                ["rm", "-f", "/vbmeta.img"]]
         },
     },
     "adb": {
