@@ -72,8 +72,42 @@ def ask_for_work_path(args):
                           " inside it! Please try again.")
 
 
-def ask_for_ui(args):
-    ui_list = pmb.helpers.ui.list(args)
+def ask_for_channel(args):
+    """ Ask for the postmarketOS release channel. The channel dictates, which
+        pmaports branch pmbootstrap will check out, and which repository URLs
+        will be used when initializing chroots.
+        :returns: channel name (e.g. "edge", "stable") """
+    channels_cfg = pmb.helpers.git.parse_channels_cfg(args)
+    count = len(channels_cfg["channels"])
+
+    # List channels
+    logging.info("Choose the postmarketOS release channel.")
+    logging.info(f"Available ({count}):")
+    for channel, channel_data in channels_cfg["channels"].items():
+        logging.info(f"* {channel}: {channel_data['description']}")
+
+    # Default for first run: "recommended" from channels.cfg
+    # Otherwise, if valid: channel from pmaports.cfg of current branch
+    # The actual channel name is not saved in pmbootstrap.cfg, because then we
+    # would need to sync it with what is checked out in pmaports.git.
+    default = pmb.config.pmaports.read_config(args)["channel"]
+    choices = channels_cfg["channels"].keys()
+    if args.is_default_channel or default not in choices:
+        default = channels_cfg["meta"]["recommended"]
+
+    # Ask until user gives valid channel
+    while True:
+        ret = pmb.helpers.cli.ask(args, "Channel", None, default,
+                                  complete=choices)
+        if ret in choices:
+            return ret
+        logging.fatal("ERROR: Invalid channel specified, please type in one"
+                      " from the list above.")
+
+
+def ask_for_ui(args, device):
+    info = pmb.parse.deviceinfo(args, device)
+    ui_list = pmb.helpers.ui.list(args, info["arch"])
     logging.info("Available user interfaces (" +
                  str(len(ui_list) - 1) + "): ")
     ui_completion_list = []
@@ -89,6 +123,22 @@ def ask_for_ui(args):
                       " one from the list above.")
 
 
+def ask_for_ui_extras(args, ui):
+    apkbuild = pmb.helpers.pmaports.get(args, "postmarketos-ui-" + ui,
+                                        subpackages=False, must_exist=False)
+    if not apkbuild:
+        return False
+
+    extra = apkbuild["subpackages"].get("postmarketos-ui-" + ui + "-extras")
+    if extra is None:
+        return False
+
+    logging.info("This user interface has an extra package: " + extra["pkgdesc"])
+
+    return pmb.helpers.cli.confirm(args, "Enable this package?",
+                                   default=args.ui_extras)
+
+
 def ask_for_keymaps(args, device):
     info = pmb.parse.deviceinfo(args, device)
     if "keymaps" not in info or info["keymaps"].strip() == "":
@@ -284,15 +334,23 @@ def ask_for_device(args):
     return (device, device_exists, kernel, nonfree)
 
 
-def ask_for_build_options(args, cfg):
-    # Allow to skip build options
-    logging.info("Build options: Parallel jobs: " + args.jobs +
-                 ", ccache per arch: " + args.ccache_size)
+def ask_for_additional_options(args, cfg):
+    # Allow to skip additional options
+    logging.info("Additional options:"
+                 f" boot partition size: {args.boot_size} MB,"
+                 f" parallel jobs: {args.jobs},"
+                 f" ccache per arch: {args.ccache_size}")
 
     if not pmb.helpers.cli.confirm(args, "Change them?",
                                    default=False):
         return
 
+    # Boot size
+    logging.info("What should be the boot partition size (in MB)?")
+    answer = pmb.helpers.cli.ask(args, "Boot size", None, args.boot_size,
+                                 validation_regex="[1-9][0-9]*")
+    cfg["pmbootstrap"]["boot_size"] = answer
+
     # Parallel job count
     logging.info("How many jobs should run parallel on this machine, when"
                  " compiling?")
@@ -349,6 +407,11 @@ def frontend(args):
     # Clone pmaports
     pmb.config.pmaports.init(args)
 
+    # Choose release channel, possibly switch pmaports branch
+    channel = ask_for_channel(args)
+    pmb.config.pmaports.switch_to_channel_branch(args, channel)
+    cfg["pmbootstrap"]["is_default_channel"] = "False"
+
     # Device
     device, device_exists, kernel, nonfree = ask_for_device(args)
     cfg["pmbootstrap"]["device"] = device
@@ -365,8 +428,10 @@ def frontend(args):
                                                      args.user, False,
                                                      "[a-z_][a-z0-9_-]*")
     # UI and various build options
-    cfg["pmbootstrap"]["ui"] = ask_for_ui(args)
-    ask_for_build_options(args, cfg)
+    ui = ask_for_ui(args, device)
+    cfg["pmbootstrap"]["ui"] = ui
+    cfg["pmbootstrap"]["ui_extras"] = str(ask_for_ui_extras(args, ui))
+    ask_for_additional_options(args, cfg)
 
     # Extra packages to be installed to rootfs
     logging.info("Additional packages that will be installed to rootfs."
