@@ -25,6 +25,7 @@ import pmb.helpers.run
 import pmb.helpers.aportupgrade
 import pmb.helpers.status
 import pmb.install
+import pmb.install.blockdevice
 import pmb.parse
 import pmb.qemu
 
@@ -117,20 +118,39 @@ def chroot(args):
             not suffix.startswith("buildroot_")):
         raise RuntimeError("--user is only supported for native or"
                            " buildroot_* chroots.")
+    if args.xauth and suffix != "native":
+        raise RuntimeError("--xauth is only supported for native chroot.")
 
     # apk: check minimum version, install packages
     pmb.chroot.apk.check_min_version(args, suffix)
     if args.add:
         pmb.chroot.apk.install(args, args.add.split(","), suffix)
 
+    # Xauthority
+    env = {}
+    if args.xauth:
+        pmb.chroot.other.copy_xauthority(args)
+        env["DISPLAY"] = os.environ.get("DISPLAY")
+        env["XAUTHORITY"] = "/home/pmos/.Xauthority"
+
+    # Install blockdevice
+    if args.install_blockdev:
+        size_boot = 128  # 128 MiB
+        size_root = 4096  # 4 GiB
+        size_reserve = 2048  # 2 GiB
+        pmb.install.blockdevice.create_and_mount_image(args, size_boot,
+                                                       size_root, size_reserve)
+
     # Run the command as user/root
     if args.user:
         logging.info("(" + suffix + ") % su pmos -c '" +
                      " ".join(args.command) + "'")
-        pmb.chroot.user(args, args.command, suffix, output=args.output)
+        pmb.chroot.user(args, args.command, suffix, output=args.output,
+                        env=env)
     else:
         logging.info("(" + suffix + ") % " + " ".join(args.command))
-        pmb.chroot.root(args, args.command, suffix, output=args.output)
+        pmb.chroot.root(args, args.command, suffix, output=args.output,
+                        env=env)
 
 
 def config(args):
@@ -140,7 +160,14 @@ def config(args):
         raise RuntimeError("Invalid config key: " + args.name)
 
     cfg = pmb.config.load(args)
-    if args.value:
+    if args.reset:
+        if args.name is None:
+            raise RuntimeError("config --reset requires a name to be given.")
+        value = pmb.config.defaults[args.name]
+        cfg["pmbootstrap"][args.name] = value
+        logging.info("Config changed to default: " + args.name + "='" + value + "'")
+        pmb.config.save(args, cfg)
+    elif args.value is not None:
         cfg["pmbootstrap"][args.name] = args.value
         logging.info("Config changed: " + args.name + "='" + args.value + "'")
         pmb.config.save(args, cfg)
@@ -177,12 +204,54 @@ def install(args):
     if args.rsync and not args.sdcard:
         raise ValueError("Installation using rsync only works on sdcard.")
 
+    # On-device installer checks
+    # Note that this can't be in the mutually exclusive group that has most of
+    # the conflicting options, because then it would not work with --sdcard.
+    if args.on_device_installer:
+        if args.full_disk_encryption:
+            raise ValueError("--on-device-installer cannot be combined with"
+                             " --fde. The user can choose to encrypt their"
+                             " installation later in the on-device installer.")
+        if args.android_recovery_zip:
+            raise ValueError("--on-device-installer cannot be combined with"
+                             " --android-recovery-zip (patches welcome)")
+        if args.no_image:
+            raise ValueError("--on-device-installer cannot be combined with"
+                             " --no-image")
+        if args.rsync:
+            raise ValueError("--on-device-installer cannot be combined with"
+                             " --rsync")
+
+    # On-device installer overrides
+    if args.on_device_installer:
+        # To make code for the on-device installer not needlessly complex, just
+        # hardcode "user" as username here. (The on-device installer will set
+        # a password for the user, disable SSH password authentication,
+        # optionally add a new user for SSH that must not have the same
+        # username etc.)
+        if args.user != "user":
+            logging.warning(f"WARNING: custom username '{args.user}' will be"
+                            " replaced with 'user' for the on-device"
+                            " installer.")
+            args.user = "user"
+
     if not args.sdcard and args.split is None:
         # Default to split if the flash method requires it
         flasher = pmb.config.flashers.get(args.deviceinfo["flash_method"], {})
         if flasher.get("split", False):
             args.split = True
 
+    # Warning for android recovery zip with FDE
+    if args.android_recovery_zip and args.full_disk_encryption:
+        logging.info("WARNING: --fde is rarely used in combination with"
+                     " --android-recovery-zip. If this does not work, consider"
+                     " using another method (e.g. installing via netcat)")
+        logging.info("WARNING: the kernel of the recovery system (e.g. TWRP)"
+                     f" must support the cryptsetup cipher '{args.cipher}'.")
+        logging.info("If you know what you are doing, consider setting a"
+                     " different cipher with 'pmbootstrap install --cipher=..."
+                     " --fde --android-recovery-zip'.")
+
     pmb.install.install(args)
 
 
@@ -240,7 +309,9 @@ def kconfig(args):
     if args.action_kconfig == "check":
         # Handle passing a file directly
         if args.file:
-            if pmb.parse.kconfig.check_file(args, args.package, details=True):
+            if pmb.parse.kconfig.check_file(args, args.package,
+                                            force_anbox_check=args.anbox,
+                                            details=True):
                 logging.info("kconfig check succeeded!")
                 return
             raise RuntimeError("kconfig check failed!")
@@ -266,7 +337,9 @@ def kconfig(args):
                 if "!pmb:kconfigcheck" in apkbuild["options"]:
                     skipped += 1
                     continue
-            if not pmb.parse.kconfig.check(args, package, details=True):
+            if not pmb.parse.kconfig.check(args, package,
+                                           force_anbox_check=args.anbox,
+                                           details=True):
                 error = True
 
         # At least one failure
