@@ -11,6 +11,7 @@ import pmb.chroot.apk
 import pmb.chroot.other
 import pmb.chroot.initfs
 import pmb.config
+import pmb.config.pmaports
 import pmb.helpers.devices
 import pmb.helpers.run
 import pmb.install.blockdevice
@@ -19,39 +20,39 @@ import pmb.install.recovery
 import pmb.install
 
 
-def mount_device_rootfs(args, suffix="native"):
+def mount_device_rootfs(args, suffix_rootfs, suffix_mount="native"):
     """
     Mount the device rootfs.
+    :param suffix_rootfs: the chroot suffix, where the rootfs that will be
+                          installed on the device has been created (e.g.
+                          "rootfs_qemu-amd64")
+    :param suffix_mount: the chroot suffix, where the device rootfs will be
+                         mounted (e.g. "native")
     """
-    mountpoint = "/mnt/rootfs_" + args.device
-    pmb.helpers.mount.bind(args, args.work + "/chroot_rootfs_" + args.device,
-                           args.work + "/chroot_" + suffix + mountpoint)
+    mountpoint = f"/mnt/{suffix_rootfs}"
+    pmb.helpers.mount.bind(args, f"{args.work}/chroot_{suffix_rootfs}",
+                           f"{args.work}/chroot_{suffix_mount}{mountpoint}")
     return mountpoint
 
 
-def get_subpartitions_size(args):
+def get_subpartitions_size(args, suffix):
     """
     Calculate the size of the boot and root subpartition.
 
+    :param suffix: the chroot suffix, e.g. "rootfs_qemu-amd64"
     :returns: (boot, root) the size of the boot and root
-              partition as integer in bytes
+              partition as integer in MiB
     """
-    # Calculate required sizes first
-    chroot = args.work + "/chroot_rootfs_" + args.device
-    root = pmb.helpers.other.folder_size(args, chroot)
-    boot = pmb.helpers.other.folder_size(args, chroot + "/boot")
-    home = pmb.helpers.other.folder_size(args, chroot + "/home")
+    boot = int(args.boot_size)
 
-    # The home folder gets omitted when copying the rootfs to
-    # /dev/installp2
-    full = root - home
-
-    # Add some free space, see also: #336, #1671
-    full *= 1.20
-    full += 50 * 1024 * 1024
-    boot *= 2
-    boot += 25 * 1024 * 1024
-    return (boot, full - boot)
+    # Estimate root partition size, then add some free space. The size
+    # calculation is not as trivial as one may think, and depending on the
+    # file system etc it seems to be just impossible to get it right.
+    chroot = f"{args.work}/chroot_{suffix}"
+    root = pmb.helpers.other.folder_size(args, chroot) / 1024 / 1024
+    root *= 1.20
+    root += 50
+    return (boot, root)
 
 
 def get_nonfree_packages(args, device):
@@ -100,16 +101,17 @@ def get_kernel_package(args, device):
     return ["device-" + device + "-kernel-" + args.kernel]
 
 
-def copy_files_from_chroot(args):
+def copy_files_from_chroot(args, suffix):
     """
     Copy all files from the rootfs chroot to /mnt/install, except
     for the home folder (because /home will contain some empty
     mountpoint folders).
+
+    :param suffix: the chroot suffix, e.g. "rootfs_qemu-amd64"
     """
     # Mount the device rootfs
-    logging.info("(native) copy rootfs_" + args.device + " to" +
-                 " /mnt/install/")
-    mountpoint = mount_device_rootfs(args)
+    logging.info(f"(native) copy {suffix} to /mnt/install/")
+    mountpoint = mount_device_rootfs(args, suffix)
     mountpoint_outside = args.work + "/chroot_native" + mountpoint
 
     # Remove empty qemu-user binary stub (where the binary was bind-mounted)
@@ -188,18 +190,19 @@ def setup_login(args):
     Loop until the password for user has been set successfully, and disable root
     login.
     """
-    # User password
-    logging.info(" *** SET LOGIN PASSWORD FOR: '" + args.user + "' ***")
     suffix = "rootfs_" + args.device
-    while True:
-        try:
-            pmb.chroot.root(args, ["passwd", args.user], suffix,
-                            output="interactive")
-            break
-        except RuntimeError:
-            logging.info("WARNING: Failed to set the password. Try it"
-                         " one more time.")
-            pass
+    if not args.on_device_installer:
+        # User password
+        logging.info(" *** SET LOGIN PASSWORD FOR: '" + args.user + "' ***")
+        while True:
+            try:
+                pmb.chroot.root(args, ["passwd", args.user], suffix,
+                                output="interactive")
+                break
+            except RuntimeError:
+                logging.info("WARNING: Failed to set the password. Try it"
+                             " one more time.")
+                pass
 
     # Disable root login
     pmb.chroot.root(args, ["passwd", "-l", "root"], suffix)
@@ -279,10 +282,11 @@ def setup_hostname(args):
         raise RuntimeError("Hostname '" + hostname + "' is not valid, please"
                            " run 'pmbootstrap init' to configure it.")
 
-    # Update /etc/hosts
     suffix = "rootfs_" + args.device
+    # Generate /etc/hostname
     pmb.chroot.root(args, ["sh", "-c", "echo " + shlex.quote(hostname) +
                            " > /etc/hostname"], suffix)
+    # Update /etc/hosts
     regex = (r"s/^127\.0\.0\.1.*/127.0.0.1\t" + re.escape(hostname) +
              " localhost.localdomain localhost/")
     pmb.chroot.root(args, ["sed", "-i", "-e", regex, "/etc/hosts"], suffix)
@@ -308,7 +312,7 @@ def embed_firmware(args):
                                "deviceinfo_sd_embed_firmware_step_size "
                                "is not valid: {}".format(step))
 
-    device_rootfs = mount_device_rootfs(args)
+    device_rootfs = mount_device_rootfs(args, f"rootfs_{args.device}")
     binaries = args.deviceinfo["sd_embed_firmware"].split(",")
 
     # Perform three checks prior to writing binaries to disk: 1) that binaries
@@ -371,31 +375,48 @@ def sanity_check_sdcard(device):
             raise RuntimeError("{} is read-only, is the sdcard locked?".format(device))
 
 
-def install_system_image(args):
-    # Partition and fill image/sdcard
-    logging.info("*** (3/5) PREPARE INSTALL BLOCKDEVICE ***")
-    pmb.chroot.shutdown(args, True)
-    (size_boot, size_root) = get_subpartitions_size(args)
-    if not args.rsync:
-        pmb.install.blockdevice.create(args, size_boot, size_root)
-        if not args.split:
-            pmb.install.partition(args, size_boot)
-    if not args.split:
-        pmb.install.partitions_mount(args)
+def sanity_check_ondev_version(args):
+    arch = args.deviceinfo["arch"]
+    package = pmb.helpers.package.get(args, "postmarketos-ondev", arch)
+    ver_pkg = package["version"].split("-r")[0]
+    ver_min = pmb.config.ondev_min_version
+    if pmb.parse.version.compare(ver_pkg, ver_min) == -1:
+        raise RuntimeError("This version of pmbootstrap requires"
+                           f" postmarketos-ondev version {ver_min} or"
+                           " higher. The postmarketos-ondev found in pmaports"
+                           f" / in the binary packages has version {ver_pkg}.")
 
-    if args.full_disk_encryption:
-        logging.info("WARNING: Full disk encryption is enabled!")
-        logging.info("Make sure that osk-sdl has been properly configured for your device")
-        logging.info("or else you will be unable to unlock the rootfs on boot!")
-        logging.info("If you started a device port, it is recommended you disable")
-        logging.info("FDE by re-running the install command without '--fde' until")
-        logging.info("you have properly configured osk-sdl. More information:")
-        logging.info("<https://postmarketos.org/osk-port>")
-    pmb.install.format(args)
+
+def install_system_image(args, size_reserve, suffix, root_label="pmOS_root",
+                         step=3, steps=5, split=False, sdcard=None):
+    """
+    :param size_reserve: empty partition between root and boot in MiB (pma#463)
+    :param suffix: the chroot suffix, where the rootfs that will be installed
+                   on the device has been created (e.g. "rootfs_qemu-amd64")
+    :param root_label: label of the root partition (e.g. "pmOS_root")
+    :param step: next installation step
+    :param steps: total installation steps
+    :param split: create separate images for boot and root partitions
+    :param sdcard: path to sdcard device (e.g. /dev/mmcblk0) or None
+    """
+    # Partition and fill image/sdcard
+    logging.info(f"*** ({step}/{steps}) PREPARE INSTALL BLOCKDEVICE ***")
+    pmb.chroot.shutdown(args, True)
+    (size_boot, size_root) = get_subpartitions_size(args, suffix)
+    if not args.rsync:
+        pmb.install.blockdevice.create(args, size_boot, size_root,
+                                       size_reserve, split, sdcard)
+        if not split:
+            pmb.install.partition(args, size_boot, size_reserve)
+    if not split:
+        root_id = 3 if size_reserve else 2
+        pmb.install.partitions_mount(args, root_id, sdcard)
+
+    pmb.install.format(args, size_reserve, root_label, sdcard)
 
     # Just copy all the files
-    logging.info("*** (4/5) FILL INSTALL BLOCKDEVICE ***")
-    copy_files_from_chroot(args)
+    logging.info(f"*** ({step + 1}/{steps}) FILL INSTALL BLOCKDEVICE ***")
+    copy_files_from_chroot(args, suffix)
     create_home_from_skel(args)
     configure_apk(args)
     copy_ssh_keys(args)
@@ -407,7 +428,7 @@ def install_system_image(args):
     if sparse is None:
         sparse = args.deviceinfo["flash_sparse"] == "true"
 
-    if sparse and not args.split and not args.sdcard:
+    if sparse and not split and not sdcard:
         logging.info("(native) make sparse rootfs")
         pmb.chroot.apk.install(args, ["android-tools"])
         sys_image = args.device + ".img"
@@ -417,8 +438,13 @@ def install_system_image(args):
         pmb.chroot.user(args, ["mv", "-f", sys_image_sparse, sys_image],
                         working_dir="/home/pmos/rootfs/")
 
-    # Kernel flash information
-    logging.info("*** (5/5) FLASHING TO DEVICE ***")
+
+def print_flash_info(args, step=5, steps=5):
+    """ Print flashing information, based on the deviceinfo data and the
+        pmbootstrap arguments.
+
+        :param step: installation step number """
+    logging.info(f"*** ({step}/{steps}) FLASHING TO DEVICE ***")
     logging.info("Run the following to flash your installation to the"
                  " target device:")
 
@@ -433,17 +459,21 @@ def install_system_image(args):
         logging.info("* pmbootstrap flasher flash_rootfs")
         logging.info("  Flashes the generated rootfs image to your device:")
         if args.split:
-            logging.info("  " + args.work + "/chroot_native/home/pmos/rootfs/" +
-                         args.device + "-rootfs.img")
+            logging.info(f"  {args.work}/chroot_native/home/pmos/rootfs/"
+                         f"{args.device}-rootfs.img")
         else:
-            logging.info("  " + args.work + "/chroot_native/home/pmos/rootfs/" +
-                         args.device + ".img")
-            logging.info("  (NOTE: This file has a partition table, which contains"
-                         " /boot and / subpartitions. That way we don't need to"
-                         " change the partition layout on your device.)")
+            logging.info(f"  {args.work}/chroot_native/home/pmos/rootfs/"
+                         f"{args.device}.img")
+            logging.info("  (NOTE: This file has a partition table, which"
+                         " contains /boot and / subpartitions. That way we"
+                         " don't need to change the partition layout on your"
+                         " device.)")
 
-    # if current flasher supports vbmeta and partition is explicitly spcified in deviceinfo
-    if "flash_vbmeta" in flasher_actions and args.deviceinfo["flash_fastboot_partition_vbmeta"]:
+    # if current flasher supports vbmeta and partition is explicitly specified
+    # in deviceinfo
+    if "flash_vbmeta" in flasher_actions and \
+            (args.deviceinfo["flash_fastboot_partition_vbmeta"] or
+             args.deviceinfo["flash_heimdall_partition_vbmeta"]):
         logging.info("* pmbootstrap flasher flash_vbmeta")
         logging.info("  Flashes vbmeta image with verification disabled flag.")
 
@@ -451,14 +481,15 @@ def install_system_image(args):
     # (e.g. an Android boot image is generated). In that case, "flash_kernel"
     # works even when partitions are split or installing for sdcard.
     # This is not possible if the flash method requires split partitions.
-    if "flash_kernel" in flasher_actions and (not requires_split or args.split):
+    if "flash_kernel" in flasher_actions and \
+            (not requires_split or args.split):
         logging.info("* pmbootstrap flasher flash_kernel")
         logging.info("  Flashes the kernel + initramfs to your device:")
         if requires_split:
-            logging.info("  " + args.work + "/chroot_native/home/pmos/rootfs/" +
-                         args.device + "-boot.img")
+            logging.info(f"  {args.work}/chroot_native/home/pmos/rootfs/"
+                         f"{args.device}-boot.img")
         else:
-            logging.info("  " + args.work + "/chroot_rootfs_" + args.device + "/boot")
+            logging.info(f"  {args.work}/chroot_rootfs_{args.device}/boot")
 
     if "boot" in flasher_actions:
         logging.info("  (NOTE: " + method + " also supports booting"
@@ -474,7 +505,7 @@ def install_system_image(args):
 def install_recovery_zip(args):
     logging.info("*** (3/4) CREATING RECOVERY-FLASHABLE ZIP ***")
     suffix = "buildroot_" + args.deviceinfo["arch"]
-    mount_device_rootfs(args, suffix)
+    mount_device_rootfs(args, f"rootfs_{args.device}", suffix)
     pmb.install.recovery.create_zip(args, suffix)
 
     # Flash information
@@ -483,13 +514,76 @@ def install_recovery_zip(args):
     logging.info("<https://postmarketos.org/recoveryzip>")
 
 
+def install_on_device_installer(args, step, steps):
+    # Generate the rootfs image
+    suffix_rootfs = f"rootfs_{args.device}"
+    install_system_image(args, 0, suffix_rootfs, step=step, steps=steps,
+                         split=True)
+    step += 2
+
+    # Prepare the installer chroot
+    logging.info(f"*** ({step}/{steps}) CREATE ON-DEVICE INSTALLER ROOTFS ***")
+    step += 1
+    packages = ([f"device-{args.device}",
+                 "postmarketos-ondev"] +
+                get_kernel_package(args, args.device) +
+                get_nonfree_packages(args, args.device))
+    suffix_installer = f"installer_{args.device}"
+    pmb.chroot.apk.install(args, packages, suffix_installer)
+
+    # Move rootfs image into installer chroot
+    img = f"{args.device}-root.img"
+    img_path_src = f"{args.work}/chroot_native/home/pmos/rootfs/{img}"
+    img_path_dest = f"{args.work}/chroot_{suffix_installer}/var/lib/rootfs.img"
+    logging.info(f"({suffix_installer}) add {img} as /var/lib/rootfs.img")
+    pmb.install.losetup.umount(args, img_path_src)
+    pmb.helpers.run.root(args, ["mv", img_path_src, img_path_dest])
+
+    # Run ondev-prepare, so it may generate nice configs from the channel
+    # properties (e.g. to display the version number), or transform the image
+    # file into another format. This can all be done without pmbootstrap
+    # changes in the postmarketos-ondev package.
+    logging.info(f"({suffix_installer}) ondev-prepare")
+    channel = pmb.config.pmaports.read_config(args)["channel"]
+    channel_cfg = pmb.config.pmaports.read_config_channel(args)
+    env = {"ONDEV_CHANNEL": channel,
+           "ONDEV_CHANNEL_BRANCH_APORTS": channel_cfg["branch_aports"],
+           "ONDEV_CHANNEL_BRANCH_PMAPORTS": channel_cfg["branch_pmaports"],
+           "ONDEV_CHANNEL_DESCRIPTION": channel_cfg["description"],
+           "ONDEV_CHANNEL_MIRRORDIR_ALPINE": channel_cfg["mirrordir_alpine"],
+           "ONDEV_CIPHER": args.cipher,
+           "ONDEV_PMBOOTSTRAP_VERSION": pmb.config.version,
+           "ONDEV_UI": args.ui}
+    pmb.chroot.root(args, ["ondev-prepare"], suffix_installer, env=env)
+
+    # Remove $DEVICE-boot.img (we will generate a new one if --split was
+    # specified, otherwise the separate boot image is not needed)
+    img_boot = f"{args.device}-boot.img"
+    logging.info(f"(native) rm {img_boot}")
+    pmb.chroot.root(args, ["rm", f"/home/pmos/rootfs/{img_boot}"])
+
+    # Generate installer image
+    size_reserve = round(os.path.getsize(img_path_dest) / 1024 / 1024) + 200
+    install_system_image(args, size_reserve, suffix_installer, "pmOS_install",
+                         step, steps, args.split, args.sdcard)
+
+
 def install(args):
     # Sanity checks
     if not args.android_recovery_zip and args.sdcard:
         sanity_check_sdcard(args.sdcard)
+    if args.on_device_installer:
+        sanity_check_ondev_version(args)
 
     # Number of steps for the different installation methods.
-    steps = 4 if args.android_recovery_zip else 5
+    if args.no_image:
+        steps = 2
+    elif args.android_recovery_zip:
+        steps = 4
+    elif args.on_device_installer:
+        steps = 8
+    else:
+        steps = 5
 
     # Install required programs in native chroot
     logging.info("*** (1/{}) PREPARE NATIVE CHROOT ***".format(steps))
@@ -508,6 +602,8 @@ def install(args):
         install_packages = [p for p in install_packages if p != "postmarketos-base"]
     if args.ui.lower() != "none":
         install_packages += ["postmarketos-ui-" + args.ui]
+        if args.ui_extras:
+            install_packages += ["postmarketos-ui-" + args.ui + "-extras"]
     suffix = "rootfs_" + args.device
     pmb.chroot.apk.upgrade(args, suffix)
 
@@ -543,7 +639,15 @@ def install(args):
     # Set the hostname as the device name
     setup_hostname(args)
 
-    if args.android_recovery_zip:
-        install_recovery_zip(args)
+    if args.no_image:
+        return
+    elif args.android_recovery_zip:
+        return install_recovery_zip(args)
+
+    if args.on_device_installer:
+        # Runs install_system_image twice
+        install_on_device_installer(args, 3, steps)
     else:
-        install_system_image(args)
+        install_system_image(args, 0, f"rootfs_{args.device}",
+                             split=args.split, sdcard=args.sdcard)
+    print_flash_info(args, steps, steps)
