@@ -258,6 +258,8 @@ def arguments_kconfig(subparser):
     check.add_argument("--arch", choices=arch_choices, dest="arch")
     check.add_argument("--file", action="store_true", help="check a file"
                        " directly instead of a config in a package")
+    check.add_argument("--anbox", action="store_true", help="check"
+                       " options needed for anbox too")
     check_package = check.add_argument("package", default="", nargs='?')
     if argcomplete:
         check_package.completer = kernel_completer
@@ -336,10 +338,13 @@ def arguments():
     # Other
     parser.add_argument("-V", "--version", action="version",
                         version=pmb.config.version)
-    parser.add_argument("-a", "--alpine-version", dest="alpine_version",
-                        help="examples: edge, latest-stable, v3.5")
     parser.add_argument("-c", "--config", dest="config",
-                        default=pmb.config.defaults["config"])
+                        default=pmb.config.defaults["config"],
+                        help="path to pmbootstrap.cfg file (default in"
+                             " ~/.config/)")
+    parser.add_argument("--config-channels",
+                        help="path to channels.cfg (which is by default"
+                             " read from pmaports.git, origin/master branch)")
     parser.add_argument("-d", "--port-distccd", dest="port_distccd")
     parser.add_argument("-mp", "--mirror-pmOS", dest="mirrors_postmarketos",
                         help="postmarketOS mirror, disable with: -mp='',"
@@ -351,6 +356,10 @@ def arguments():
                              pmb.config.defaults["mirror_alpine"],
                         metavar="URL")
     parser.add_argument("-j", "--jobs", help="parallel jobs when compiling")
+    parser.add_argument("-B", "--boot-size",
+                        help="specify an integer with your preferred boot"
+                             "partition size on target machine in MB (default"
+                             " 128)")
     parser.add_argument("-p", "--aports",
                         help="postmarketos aports (pmaports) path")
     parser.add_argument("-t", "--timeout", help="seconds after which processes"
@@ -371,7 +380,7 @@ def arguments():
                         " the package index files", action="store_true")
 
     # Compiler
-    parser.add_argument("--ccache-disable", action="store_false",
+    parser.add_argument("--no-ccache", action="store_false",
                         dest="ccache", help="do not cache the compiled output")
     parser.add_argument("--no-crossdirect", action="store_true",
                         help="Don't use the new, faster 'crossdirect' method,"
@@ -484,6 +493,14 @@ def arguments():
                         default="tui")
     chroot.add_argument("command", default=["sh", "-i"], help="command"
                         " to execute inside the chroot. default: sh", nargs='*')
+    chroot.add_argument("-x", "--xauth", action="store_true",
+                        help="Copy .Xauthority and set environment variables,"
+                             " so X11 applications can be started (native"
+                             " chroot only)")
+    chroot.add_argument("-i", "--install-blockdev", action="store_true",
+                        help="Create a sparse image file and mount it as"
+                              " /dev/install, just like during the"
+                              " installation process.")
     for action in [build_init, chroot]:
         suffix = action.add_mutually_exclusive_group()
         if action == chroot:
@@ -512,6 +529,8 @@ def arguments():
     group.add_argument("--android-recovery-zip",
                        help="generate TWRP flashable zip",
                        action="store_true", dest="android_recovery_zip")
+    group.add_argument("--no-image", help="do not generate the image",
+                       action="store_true", dest="no_image")
     install.add_argument("--rsync", help="update the sdcard using rsync,"
                          " does not work with --fde", action="store_true")
     install.add_argument("--cipher", help="cryptsetup cipher used to"
@@ -538,6 +557,11 @@ def arguments():
     install.add_argument("--no-base",
                          help="do not install postmarketos-base (advanced)",
                          action="store_false", dest="install_base")
+    install.add_argument("--on-device-installer", "--ondev",
+                         action="store_true",
+                         help="wrap the resulting image in a graphical"
+                              " on-device installer, so the installation can"
+                              " be customized after flashing")
     group = install.add_mutually_exclusive_group()
     group.add_argument("--sparse", help="generate sparse image file"
                        " (even if unsupported by device)", default=None,
@@ -603,6 +627,8 @@ def arguments():
     # Action: config
     config = sub.add_parser("config",
                             help="get and set pmbootstrap options")
+    config.add_argument("-r", "--reset", action="store_true",
+                        help="Reset config options with the given name to it's default.")
     config.add_argument("name", nargs="?", help="variable name, one of: " +
                         ", ".join(sorted(pmb.config.config_keys)),
                         choices=pmb.config.config_keys, metavar="name")
