@@ -17,7 +17,8 @@ import pmb.helpers.logging
 @pytest.fixture
 def args(tmpdir, request):
     import pmb.parse
-    sys.argv = ["pmbootstrap.py", "chroot"]
+    cfg = f"{pmb_test.const.testdata}/channels.cfg"
+    sys.argv = ["pmbootstrap.py", "--config-channels", cfg, "chroot"]
     args = pmb.parse.arguments()
     args.log = args.work + "/log_testsuite.txt"
     args.fork_alpine = False
@@ -74,17 +75,19 @@ def test_aportgen_fork_alpine_compare_output(args, tmpdir, monkeypatch):
 
 def test_aportgen(args, tmpdir):
     # Fake aports folder in tmpdir
+    testdata = pmb_test.const.testdata
     tmpdir = str(tmpdir)
     shutil.copytree(args.aports + "/.git", tmpdir + "/.git")
     args.aports = tmpdir
+    shutil.copy(f"{testdata}/pmaports.cfg", args.aports)
     os.mkdir(tmpdir + "/cross")
 
     # Create aportgen folder -> code path where it still exists
     pmb.helpers.run.user(args, ["mkdir", "-p", args.work + "/aportgen"])
 
     # Generate all valid packages (gcc twice -> different code path)
-    pkgnames = ["binutils-armhf", "musl-armhf", "busybox-static-armhf",
-                "gcc-armhf", "gcc-armhf"]
+    pkgnames = ["binutils-armv7", "musl-armv7", "busybox-static-armv7",
+                "gcc-armv7", "gcc-armv7"]
     for pkgname in pkgnames:
         pmb.aportgen.generate(args, pkgname)
 
